#include <stdio.h>

#include "xparameters.h"
#include "xio.h"

#include "evoBlock.h"

unsigned int memoryTest(Xuint32 startAddr, Xuint32 endAddr, unsigned int maxErrors, unsigned int infoCount);
void evoArrayTest(Xuint32);

int main(void)
{
	xil_printf("System Test\n-----------\n");

	evoArrayTest(XPAR_PLB_IFCE_EVOBLOCK_0_BASEADDR);
	memoryTest(XPAR_DDR_SDRAM_MPMC_BASEADDR, XPAR_DDR_SDRAM_MPMC_HIGHADDR, 10, 2048000);

	return 0;
}

void evoArrayTest(Xuint32 baseAddr)
{
	int i;
	Xuint32 val;
	xil_printf("Evo Array: 0x%08X\n", baseAddr);

	xil_printf("Basic read/write test\n");
	EVOBLOCK_writeTestReg(0xA1B2C3D4);
	val = EVOBLOCK_readTestReg();
	xil_printf(" - Done: 0x%08X\n", val);

	xil_printf("LED test\n");
	val = 0;
	for (i=0 ; i<16 ; i++) {
		Xuint32 delay = 0xFFFFF;
		while (delay--) asm volatile ("nop");
		EVOBLOCK_writeTestReg(val);
		val ++;
	}
	xil_printf(" - Done\n");
}

unsigned int memoryTest(Xuint32 startAddr, Xuint32 endAddr, unsigned int maxErrors, unsigned int infoCount)
{
	const Xuint32 seed = 0x12345678;
	const Xuint32 mask = 0x80000062;

	if (infoCount != 0) xil_printf("Memory Test: 0x%08X to 0x%08X\n", startAddr, endAddr);

	// Fill RAM
	Xuint32 rand = seed;
	Xuint32 addr = startAddr;
	unsigned int count = 0;
	unsigned int totalCount = 0;

	if (infoCount != 0) xil_printf("Filling memory:\n");

	while (addr <= endAddr) {
		XIo_Out32(addr, rand);

		// Create next value
		if (rand & 1) rand = (1 << 31) | ((rand ^ mask) >> 1);
		else rand >>= 1;

		count ++;
		totalCount ++;
		if (infoCount != 0 && count == infoCount) {
			xil_printf("%d\n", totalCount);
			count = 0;
		}

		addr += 4;
	}

	if (infoCount != 0) xil_printf("Done\n");

	// Read RAM
	rand = seed;
	addr = startAddr;
	count = 0;
	totalCount = 0;
	unsigned int errors = 0;

	if (infoCount != 0) xil_printf("Reading memory:\n");

	while (addr <= endAddr) {
		Xuint32 val = XIo_In32(addr);

		if (val != rand) {
			errors++;
			xil_printf("Error - Address: 0x%08X, Written: 0x%08X, Read: 0x%08X\n", addr, rand, val);
		}

		if (errors == maxErrors) break;

		// Create next value
		if (rand & 1) rand = (1 << 31) | ((rand ^ mask) >> 1);
		else rand >>= 1;

		count ++;
		totalCount ++;
		if (infoCount != 0 && count == infoCount) {
			xil_printf("%d\n", totalCount);
			count = 0;
		}

		addr += 4;
	}

	if (infoCount != 0) xil_printf("Done\n");

	xil_printf("%d error%s found\n", errors, (errors==1) ? "" : "s");

	return errors;
}
